/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included  with this distribution in *
 * the LICENSE file.                                                         *
 *****************************************************************************/
/*****************************************************************************
 * @author <a href="mailto:pankaj_kumar@hp.com">Pankaj Kumar</a>             *
 *         (Hewlett-Packard Company)                                         *
 * Date Originally Created: July 18 - 22, 2001                               *
 * @version CVS $Revision$ $Date$                                            *
 *****************************************************************************/
package org.apache.cocoon.tools.cpp.gui;

import java.io.File;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;

import java.awt.Dimension;
import java.awt.Toolkit;


import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.KeyEvent;

import javax.swing.JInternalFrame;
import javax.swing.JDesktopPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JSeparator;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import javax.swing.UIManager;
import javax.swing.text.html.HTMLDocument;


public class CPPServer extends JFrame
  {
  private static JDesktopPane desktop;
  public static int desktopWidth, desktopHeight;
  private static CPPViewer currentCPPViewer = null;
  private final JFileChooser fc = new JFileChooser();

  public CPPServer()
    {
    super("CPPServer");

    // Set Platform Look & Feel
    try
      {
	  UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      }
    catch (Exception e)
      {
	  System.err.println("Cannot change Look & Feel. Exception: " + e);
      }

    //Make the big window be indented 50 pixels from each edge
    //of the screen.
    int inset = 50;
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    desktopWidth = screenSize.width - inset*2;
    desktopHeight = screenSize.height-inset*2;
    setBounds(inset, inset, desktopWidth, desktopHeight);

    //Quit this app when the big window closes.
    addWindowListener(new WindowAdapter()
      {
      public void windowClosing(WindowEvent e)
        {
		System.out.println("Main Window closing ...");
        System.exit(0);
        }
      });

    //Set up the GUI.
    desktop = new JDesktopPane(); //a specialized layered pane
    desktop.setBackground(CPPConstants.DESKTOP_COLOR);
    setContentPane(desktop);
    setJMenuBar(createMenuBar());

    //Make dragging faster:
    desktop.putClientProperty("JDesktopPane.dragMode", "outline");
    }

  protected JMenuBar createMenuBar()
    {
    JMenuBar menuBar = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);

    // Menu Item: Open -- Opens a previously saved file.
    JMenuItem openMenuItem = new JMenuItem("Open");
    openMenuItem.setMnemonic(KeyEvent.VK_N);
    openMenuItem.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
		int returnVal = fc.showOpenDialog(CPPServer.this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		  {
		  File file = fc.getSelectedFile();
		  System.out.println("Opening File: " + file.getAbsolutePath());
          CPPViewer frame = createFrame(file.getAbsolutePath());
          frame.readFromFile(file);
	      }
        }
      });
    fileMenu.add(openMenuItem);

    // Menu Item: Save -- Save the contents to the current file.
    JMenuItem saveMenuItem = new JMenuItem("Save");
    saveMenuItem.setMnemonic(KeyEvent.VK_S);
    saveMenuItem.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
		File file = currentCPPViewer.getFile();
		if (!file.exists())	// Need to show the file chooser dialog
		  {
		  int returnVal = fc.showSaveDialog(CPPServer.this);
		  if (returnVal == JFileChooser.APPROVE_OPTION)
		    file = fc.getSelectedFile();
		  else
		    return;	// do nothing.
	      }
	    System.out.println("Saving File: " + file.getAbsolutePath());
	    CPPServer.currentCPPViewer.writeToFile(file);
        }
      });
    fileMenu.add(saveMenuItem);

    // Menu Item: SaveAs -- Save the contents to a user specified file.
    JMenuItem saveAsMenuItem = new JMenuItem("Save As");
    saveAsMenuItem.setMnemonic(KeyEvent.VK_A);
    saveAsMenuItem.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
		File file = null;
		int returnVal = fc.showSaveDialog(CPPServer.this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
		  file = fc.getSelectedFile();
		else
		  return;	// do nothing.
		System.out.println("Saving File: " + file.getAbsolutePath());
	    CPPServer.currentCPPViewer.writeToFile(file);
        }
      });
    fileMenu.add(saveAsMenuItem);

    fileMenu.add(new JSeparator());

    // Menu Item: exit -- Save the contents to a user specified file.
    JMenuItem exitMenuItem = new JMenuItem("Exit");
    exitMenuItem.setMnemonic(KeyEvent.VK_X);
    exitMenuItem.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
		System.out.println("Normal Exit ...");
		System.exit(0);
        }
      });
    fileMenu.add(exitMenuItem);

    menuBar.add(fileMenu);

    JMenu helpMenu = new JMenu("Help");
    helpMenu.setMnemonic(KeyEvent.VK_H);

    // Menu Item: Help Window -- Opens a previously saved file.
    JMenuItem helpMenuItem = new JMenuItem("Help Window");
    helpMenuItem.setMnemonic(KeyEvent.VK_H);
    helpMenuItem.addActionListener(new ActionListener()
      {
      public void actionPerformed(ActionEvent e)
        {
		System.out.println("ActionEvent for Help Window menu item ...");
	    File file = new File("help.html");
	    CPPViewer frame = createFrame(file.getAbsolutePath(),
	    	CPPConstants.HELPW_WIDTH,
	        CPPConstants.HELPW_HEIGHT,
	        desktopWidth - CPPConstants.HELPW_WIDTH,
	        0);
	    frame.readFromFile(file);
	    }
      });
    helpMenu.add(helpMenuItem);
    helpMenu.add(new JSeparator());

    // Menu Item: About CPP Window -- Opens a previously saved file.
    JMenuItem aboutMenuItem = new JMenuItem("About CPP");
    aboutMenuItem.setMnemonic(KeyEvent.VK_C);
    aboutMenuItem.addActionListener(new ActionListener()
	  {
	  public void actionPerformed(ActionEvent e)
	    {
		System.out.println("ActionEvent for About CPP menu item ...");
	    File file = new File("about.html");
	    CPPViewer frame = createFrame(file.getAbsolutePath(),
	    	CPPConstants.ABOUTW_WIDTH,
	    	CPPConstants.ABOUTW_HEIGHT,
	   		desktopWidth/2 - CPPConstants.ABOUTW_WIDTH/2,
	   		desktopHeight/2 - CPPConstants.ABOUTW_HEIGHT/2);
	    frame.readFromFile(file);
	    }
      });
    helpMenu.add(aboutMenuItem);
    menuBar.add(helpMenu);

    return menuBar;
    }

  public static CPPViewer createFrame(String name)
    {
    CPPViewer frame = new CPPViewer(name);
    addFrame(frame);
    return frame;
    }

  public static CPPViewer createFrame(String name, int width, int height, int x, int y)
    {
    CPPViewer frame = new CPPViewer(name, width, height, x, y);
    addFrame(frame);
    return frame;
    }

  public static void addFrame(CPPViewer frame)
    {
    frame.setVisible(true); //necessary as of kestrel
    desktop.add(frame);
    try
      {
      frame.setSelected(true);
      }
    catch (java.beans.PropertyVetoException e)
      {
	  System.err.println("CPPServer:JInternalFrame.setSelected() failed. Exception: " + e);
	  System.exit(1);
	  }
    }

  public static void setCurrentCPPViewer(CPPViewer viewer)
    {
	currentCPPViewer = viewer;
    }

  public static void main(String[] args) throws IOException
    {
    int listenPort = CPPConstants.LISTEN_PORT;
    ServerSocket serverSocket = null;
    boolean listening = true;

    // Create socket to listen for connection from CPPTransformer
    try
      {
      serverSocket = new ServerSocket(listenPort);
      }
    catch (IOException e)
      {
      System.err.println("Could not listen on port: " + listenPort + ". Exception: " + e);
      System.exit(1);
      }


    CPPServer frame = new CPPServer();
    frame.setVisible(true);

    while (listening)
      {
	  Socket socket = serverSocket.accept();
	  System.out.println("Connection Accepted. Socket: " + socket);
	  new CPPServerThread(socket).start();
      }

    serverSocket.close();
    }
  }
