/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included  with this distribution in *
 * the LICENSE file.                                                         *
 *****************************************************************************/
/*****************************************************************************
 * @author <a href="mailto:pankaj_kumar@hp.com">Pankaj Kumar</a>             *
 *         (Hewlett-Packard Company)                                         *
 * Date Originally Created: July 18 - 22, 2001                               *
 * @version CVS $Revision$ $Date$                                            *
 *****************************************************************************/
package org.apache.cocoon.tools.cpp.transformer;

import java.util.HashMap;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.io.IOException;
import java.net.Socket;
import java.net.UnknownHostException;

import org.apache.log.Logger;
import org.apache.log.Hierarchy;

class CPPChannelManager
  {
  private static Logger log = null;
  private static HashMap channels = new HashMap();


  private static class NullWriter extends Writer
    {
	public NullWriter() { }
	public void write(char[] cbuf, int off, int n) throws IOException
	  {
	  throw new IOException("NullWriter -- indicate error");
	  }
	public void flush() throws IOException
	  {
	  throw new IOException("NullWriter -- indicate error");
	  }
	public void close() throws IOException
	  {
	  throw new IOException("NullWriter -- indicate error");
	  }
    }

  public static synchronized CPPChannel get(String name, String hostName, int hostPort)
	{
	if (log == null)
	  log = Hierarchy.getDefaultHierarchy().getLoggerFor("cocoon");

	CPPChannel chan = (CPPChannel)channels.get(name);
	if (chan != null)	// An entry found.
	  {
	  if (chan.isActive())	// Only means that the last write was successful !!
	    {
		if (chan.testActivity())
		  {
          log.debug("CPPChannelManager: Reusing channel for CPPTransformer with name: " + name);
	      return chan;
	      }
	    }
      log.debug("CPPChannelManager: Removing inactive channel for CPPTransformer with name: " + name);
	  channels.remove(name);
      }

    // A new CPPChannel needs to be created.
    OutputStream os = null;
    try
	  {
	  Socket socket = new Socket(hostName, hostPort);
	  os = socket.getOutputStream();
	  }
    catch (UnknownHostException e)
	  {
      log.debug("CPPChannelManager: Unknown host: " + hostName);
	  }
    catch (IOException e)
	  {
	  log.debug("CPPChannelManager: Connection failed with host: " + hostName + " at port: " + hostPort);
      }

    Writer wr = null;
    if (os == null)
      {
	  wr = (Writer)new CPPChannelManager.NullWriter();
	  log.debug("CPPChannelManager: Creating dummy channel for CPPTransformer with name: " + name);
      }
    else
      {
	  wr = (Writer)new OutputStreamWriter(os);
	  log.debug("CPPChannelManager: Creating channel for CPPTransformer with name: " + name);
      }
    chan = new CPPChannel(wr);	// Could return an inactive channel
    if (chan.isActive())
      chan.println(name);	// Name to the server
    channels.put(name, chan);
    return chan;
	}
  }
