/*****************************************************************************
 * Copyright (C) The Apache Software Foundation. All rights reserved.        *
 * ------------------------------------------------------------------------- *
 * This software is published under the terms of the Apache Software License *
 * version 1.1, a copy of which has been included  with this distribution in *
 * the LICENSE file.                                                         *
 *****************************************************************************/
/*****************************************************************************
 * @author <a href="mailto:pankaj_kumar@hp.com">Pankaj Kumar</a>             *
 *         (Hewlett-Packard Company)                                         *
 * Date Originally Created: July 18 - 22, 2001                               *
 * @version CVS $Revision$ $Date$                                            *
 *****************************************************************************/
package org.apache.cocoon.tools.cpp.gui;

import java.awt.Dimension;
import java.awt.Toolkit;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;

import javax.swing.JInternalFrame;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JInternalFrame;

import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLDocument;

public class CPPViewer extends JInternalFrame
  {
  private File file = null;

  static final int xOffset = 30, yOffset = 30;
  static  int curX = 2*xOffset, curY = yOffset;
  private HTMLEditorKit kit = null;
  private HTMLDocument doc = null;

  public CPPViewer(String name)
    {
	this(name, CPPConstants.CPPVIEWER_WIDTH, CPPConstants.CPPVIEWER_HEIGHT, -1, -1);
    }

  public CPPViewer(String name, int width, int height, int x, int y)
    {
    super("CPPViewer -- " + name,
              true, //resizable
              true, //closable
              true, //maximizable
              true);//iconifiable
    file = new File(name);

    addInternalFrameListener(new InternalFrameAdapter()
      {
      public void internalFrameActivated(InternalFrameEvent e)
        {
		CPPServer.setCurrentCPPViewer(CPPViewer.this);
        }
      });

    //Create an editor pane.
    JEditorPane editorPane = createEditorPane();
    JScrollPane editorScrollPane = new JScrollPane(editorPane);
    editorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    editorScrollPane.setPreferredSize(new Dimension(200, 200));
    editorScrollPane.setMinimumSize(new Dimension(10, 10));

    getContentPane().add(editorScrollPane);

    //...Then set the window size or call pack...
    setSize(width, height);
    if (x == -1 && y == -1)		// determine location
      {
	  curX = curX + xOffset;
	  curY = curY + yOffset;
	  if ((curX + width > CPPServer.desktopWidth) || (curY + height > CPPServer.desktopHeight))
	    {
        curX = 0;
        curY = 0;
	    }
      setLocation(curX, curY);
      }
    else
      setLocation(x, y);
    }


  public HTMLEditorKit getEditorKit()	{ return kit; }
  public HTMLDocument getDocument()		{ return doc; }
  public File getFile()				{ return file; }
  public void setFile(File file)	{ this.file = file; }

  private JEditorPane createEditorPane()
    {
    JEditorPane editorPane = new JEditorPane();
    editorPane.setEditable(false);

    kit = new HTMLEditorKit();
 	doc = (HTMLDocument) (kit.createDefaultDocument());

    editorPane.setEditorKit(kit);
    editorPane.setDocument(doc);
    editorPane.setContentType("text/html");

    return editorPane;
    }

  public void readFromFile(File f)
    {
    try
      {
      BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(f)));

	  HTMLDocument doc = this.getDocument();
	  this.getEditorKit().read(in, doc, doc.getLength());
	  in.close();
	  }
	catch (Exception ex)
	  {
	  System.err.println("CPPServer:file read failed. Exception: " + ex);
	  }
    }

  public void writeToFile(File f)
    {
    try
      {
      BufferedWriter out = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(f)));

	  HTMLDocument doc = this.getDocument();
	  this.getEditorKit().write(out, doc, 0, doc.getLength());
	  out.close();
	  }
	catch (Exception ex)
	  {
	  System.err.println("CPPServer:file write failed. Exception: " + ex);
	  }
	}
  }
